/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_update_subject.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Party/Subject creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_update_subject(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2012-2013 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_update_subject(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   New Subject Updates -                                   *
	 *                   iem.new_subject_updates                                * 
	 *                  PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 * Target Table:    Join - work.stg_subject_upd                             *
	 *==========================================================================*/ 

	data _null_;
		call symput('current_dt',put(datetime(),datetime22.));
	run;

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete stg_subject_upd;
	quit;

	proc sql;
	   create table work.stg_subject_upd as
	   select
	      PARTY_LIVE.PARTY_RK,
	      ("&current_dt"dt) as VALID_FROM_DTTM,
	      PARTY_LIVE.VALID_TO_DTTM,
	      new_subject_updates.PARTY_ID,
	      new_subject_updates.SOURCE_SYSTEM_CD,
	      PARTY_LIVE.PARTY_TYPE_CD,
	      PARTY_LIVE.PARTY_CATEGORY_CD,
	      PARTY_LIVE.PARTY_SUBCATEGORY_CD,
	      PARTY_LIVE.IDENTICAL_PARTY_LINK_SK,
	      PARTY_LIVE.INDIVIDUAL_FLG,
	      new_subject_updates.PARTY_FULL_NM,
	      PARTY_LIVE.NATIONAL_ID,
	      PARTY_LIVE.NATIONAL_ID_TYPE_CD,
	      PARTY_LIVE.UI_DEF_FILE_NM,
	      PARTY_LIVE.CREATE_USER_ID,
	      PARTY_LIVE.CREATE_DTTM,
	      PARTY_LIVE.UPDATE_USER_ID,
	      PARTY_LIVE.VERSION_NO,
	      PARTY_LIVE.DELETE_FLG,
	      new_subject_updates.X_SASFCS_PTY_KEY,
	      new_subject_updates.X_SASFCS_RSK_CLSSF,
	      new_subject_updates.X_SASFCS_TIN,
	      PARTY_LIVE.VALID_FROM_DTTM as OLD_VALID_FROM_DTTM
	   from
	      iem.new_subject_updates, 
	      ecm_db.PARTY_LIVE
	   where
	      new_subject_updates.PARTY_ID = PARTY_LIVE.PARTY_ID
	      and new_subject_updates.SOURCE_SYSTEM_CD = PARTY_LIVE.SOURCE_SYSTEM_CD
	   ;
	quit;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Update                                                  *
	 * Transform:       Update                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.stg_subject_upd                             *
	 * Target Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          *
	 *                                                                          * 
	 * User Written:    SourceCode                                              * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.stg_subject_upd); 

	proc sql;
	   update ecm_db.PARTY_LIVE
	      set 
	         VALID_FROM_DTTM= (
	            select
	               stg_subject_upd.VALID_FROM_DTTM
	            from
	               work.stg_subject_upd as stg_subject_upd
	            where
	               PARTY_LIVE.PARTY_RK = stg_subject_upd.PARTY_RK
	         ),
	         VERSION_NO = VERSION_NO+1
	      where
	         PARTY_RK IN (
	            select
	               stg_subject_upd.PARTY_RK
	            from
	               work.stg_subject_upd as stg_subject_upd
	         )
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Update                                                  *
	 * Transform:       Update                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.stg_subject_upd                             *
	 * Target Table:    PARTY_VERSION - ecm_db.PARTY_VERSION                    *
	 *                                                                          * 
	 * User Written:    SourceCode                                              * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.stg_subject_upd); 

	proc sql;
	   update ecm_db.PARTY_VERSION
	      set 
	         VALID_TO_DTTM= (
	            select
	               stg_subject_upd.VALID_FROM_DTTM
	            from
	               work.stg_subject_upd as stg_subject_upd
	            where
	               PARTY_VERSION.PARTY_RK = stg_subject_upd.PARTY_RK
	         )
	      where
	         VALID_TO_DTTM = .
	         and PARTY_RK IN (
	            select
	               stg_subject_upd.PARTY_RK
	            from
	               work.stg_subject_upd as stg_subject_upd
	         )
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.stg_subject_upd                             *
	 * Target Table:    PARTY_VERSION - ecm_db.PARTY_VERSION                    *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.stg_subject_upd); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	  delete temp_map;
	quit;

	proc sql;
	  create view work.temp_map as
	     select
	        PARTY_RK,
	        VALID_FROM_DTTM,
	        VALID_TO_DTTM,
	        PARTY_ID,
	        SOURCE_SYSTEM_CD,
	        PARTY_TYPE_CD,
	        PARTY_CATEGORY_CD,
	        PARTY_SUBCATEGORY_CD,
	        IDENTICAL_PARTY_LINK_SK,
	        INDIVIDUAL_FLG,
	        PARTY_FULL_NM,
	        NATIONAL_ID,
	        NATIONAL_ID_TYPE_CD,
	        UI_DEF_FILE_NM,
	        CREATE_USER_ID,
	        CREATE_DTTM,
	        UPDATE_USER_ID,
	        (VERSION_NO+1) as VERSION_NO,
	        DELETE_FLG
	  from &SOURCEDS
	  ;
	quit;

	%let SOURCEDS = work.temp_map;

	proc append base = ecm_db.PARTY_VERSION 
	  data = &SOURCEDS force ; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	proc datasets lib = work nolist nowarn memtype = (data view);
	  delete temp_map;
	quit;

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.stg_subject_upd                             *
	 * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.stg_subject_upd); 

	data _null_;
		call symput('current_dt',put(datetime(),datetime22.));
	run;

	proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

	data iem.new_events;
		if 0 then set ecm_db.ecm_event;
		set &SOURCEDS. (keep= PARTY_RK);
		BUSINESS_OBJECT_NM = "PARTY";
		BUSINESS_OBJECT_RK = PARTY_RK;
		EVENT_TYPE_CD = "SAVENT";
		CREATE_USER_ID = "&ecm_user";
		CREATE_DTTM = "&current_dt"dt;
		drop PARTY_RK;
	run;
	
    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_EVENTS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let obs_count = %fcf_get_observation_count(table=iem.new_events,SASDATASET=Y);
    %let numevents = &obs_count;

    %fcf_get_ecm_sequence_ids(numIDs=&numevents, ecmSeq=event);

    data iem.new_events;
        set iem.new_events;
        event_rk = symgetn('seqId'||(kstrip(put(_n_,best32.))));
    run;

    proc append base = ecm_db.ECM_EVENT 
      data = iem.new_events force ; 
    run; 

	%fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to append to ECM_EVENT for Subject update;

        %fcf_put_error;
        %goto MACRO_END;
    %end;


	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.stg_subject_upd                             *
	 * Target Table:    Character UDFs - work.character_udfs                    *
	 *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.stg_subject_upd); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete character_udfs;
	quit;

	proc sql;
	   create view work.character_udfs as
	      select
	         PARTY_RK as ECM_RK,
	         VALID_FROM_DTTM,
	         X_SASFCS_TIN
	   from &SOURCEDS
	   ;
	quit;

	%let SOURCEDS = work.character_udfs;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Sort                                                    *
	 * Transform:       Sort                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Character UDFs - work.character_udfs                    *
	 * Target Table:    Sorted Character UDFs -                                 *
	 *                   work.sorted_character_udfs                             * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.character_udfs); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete sorted_character_udfs;
	quit;

	proc sort data = &SOURCEDS out = work.sorted_character_udfs; 
	   by 
	      ECM_RK
	      VALID_FROM_DTTM
	   ;
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Transpose                                               *
	 * Transform:       Transpose                                               * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Sorted Character UDFs -                                 *
	 *                   work.sorted_character_udfs                             * 
	 * Target Table:    Transposed Character UDFs -                             *
	 *                   work.transposed_character_udfs                         * 
	 *==========================================================================*/ 

    %let _INPUT = work.sorted_character_udfs;
    %let _OUTPUT = work.transposed_character_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

	proc datasets lib=work nolist nowarn memtype = (data view);
	   delete transposed_character_udfs;
	quit;

	%let varColumns = X_SASFCS_TIN;
	%let byColumns = ECM_RK VALID_FROM_DTTM;

	proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
		by &byColumns;
		var &varColumns;
 	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end;
	
	/*==========================================================================* 
	 * Step:            Data Validation                                         *
	 * Transform:       Data Validation                                         * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Transposed Character UDFs -                             *
	 *                   work.transposed_character_udfs                         * 
	 * Target Table:    Validated Character UDFs -                              *
	 *                   work.validated_character_udfs                          * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.transposed_character_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete staged_character_udfs;
    quit;

    data work.validated_character_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        if ( _NAME_ = "X_SASFCS_TIN"  & COL1 = '') then
            do;
                i_move = 1;
            end;  /* if ( _NAME_ = "X_SASFCS_TIN"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /* move record to error file  */
        if (i_move) then
            output work.temp_errors;

        /* move record to output file  */
        else output work.validated_character_udfs;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate TRANSPOSED_CHARACTER_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Validated Character UDFs -                              *
	 *                   work.validated_character_udfs                          * 
	 * Target Table:    New Character UDFs -                                    *
	 *                   work.new_character_udfs                                * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.validated_character_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_character_udfs;
    quit;

	data new_character_udfs;
		if 0 then set ecm_db.party_udf_char_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		PARTY_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "PARTY";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

    %let SOURCEDS = work.new_character_udfs;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_UDF_CHAR_VALUE -                                  *
	 *                   ecm_db.PARTY_UDF_CHAR_VALUE                            * 
	 *                  Join - work.stg_subject_upd                             *
	 * Target Table:    All non-FCF Character UDFs -                            *
	 *                   work.non_fcf_char_udfs                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete non_fcf_char_udfs;
	quit;

	proc sql;
	   create view work.non_fcf_char_udfs as
	   select
	      PARTY_UDF_CHAR_VALUE.PARTY_RK,
	      stg_subject_upd.VALID_FROM_DTTM,
	      PARTY_UDF_CHAR_VALUE.UDF_TABLE_NM,
	      PARTY_UDF_CHAR_VALUE.UDF_NM,
	      PARTY_UDF_CHAR_VALUE.ROW_NO,
	      PARTY_UDF_CHAR_VALUE.UDF_VALUE
	   from
	      work.stg_subject_upd, 
	      ecm_db.PARTY_UDF_CHAR_VALUE
	   where
	      stg_subject_upd.PARTY_RK = PARTY_UDF_CHAR_VALUE.PARTY_RK
	      and stg_subject_upd.OLD_VALID_FROM_DTTM = PARTY_UDF_CHAR_VALUE.VALID_FROM_DTTM
	      and PARTY_UDF_CHAR_VALUE.UDF_NM NOT IN ('X_SASFCS_TIN')
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Append                                                  *
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   All non-FCF Character UDFs -                            *
	 *                   work.non_fcf_char_udfs                                 * 
	 *                  New Character UDFs -                                    *
	 *                   work.new_character_udfs                                * 
	 * Target Table:    All Character UDFs -                                    *
	 *                   work.all_character_udfs                                * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_character_udfs;
	quit;

	data work.all_character_udfs / view=work.all_character_udfs;
	   set work.non_fcf_char_udfs
	       work.new_character_udfs;
	   keep PARTY_RK VALID_FROM_DTTM UDF_TABLE_NM UDF_NM ROW_NO UDF_VALUE; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 


	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Character UDFs -                                    *
	 *                   work.all_character_udfs                                * 
	 * Target Table:    PARTY_UDF_CHAR_VALUE -                                  *
	 *                   ecm_db.PARTY_UDF_CHAR_VALUE                            * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_character_udfs); 

	proc append base = ecm_db.PARTY_UDF_CHAR_VALUE 
    	data = &SOURCEDS force ; 
    run; 
	   
	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
		
	/*==========================================================================* 
	 * Step:            Extract                                                 *
	 * Transform:       Extract                                                 * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Join - work.stg_subject_upd                             *
	 * Target Table:    Numeric UDFs - work.numeric_udfs                        *
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.stg_subject_upd); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete numeric_udfs;
	quit;

	proc sql;
	   create view work.numeric_udfs as
	      select
	         PARTY_RK as ECM_RK,
	         VALID_FROM_DTTM,
	         X_SASFCS_RSK_CLSSF,
	         X_SASFCS_PTY_KEY
	   from &SOURCEDS
	   ;
	quit;

	%let SOURCEDS = work.numeric_udfs;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Sort                                                    *
	 * Transform:       Sort                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Numeric UDFs - work.numeric_udfs                        *
	 * Target Table:    Sorted Numeric UDFs -                                   *
	 *                   work.sorted_numeric_udfs                               * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.numeric_udfs); 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete sorted_numeric_udfs;
	quit;

	proc sort data = &SOURCEDS out = work.sorted_numeric_udfs; 
	   by
	      ECM_RK
	      VALID_FROM_DTTM
	   ;
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Transpose                                               *
	 * Transform:       Transpose                                               * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Sorted Numeric UDFs -                                   *
	 *                   work.sorted_numeric_udfs                               * 
	 * Target Table:    Transposed Numeric UDFs -                               *
	 *                   work.transposed_numeric_udfs                           * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.sorted_numeric_udfs); 

	%let _INPUT = work.sorted_numeric_udfs;
	%let _OUTPUT = work.transposed_numeric_udfs;
	%let _OUTPUT_options = %nrquote(where = (n(ecm_rk)>0));

	proc datasets lib=work nolist nowarn memtype = (data view);
	   delete transposed_numeric_udfs;
	quit;

	%let varColumns = X_SASFCS_RSK_CLSSF X_SASFCS_PTY_KEY;
	%let byColumns = ECM_RK VALID_FROM_DTTM;

	proc transpose data = &_INPUT out = &_OUTPUT (&_OUTPUT_options);
	      by &byColumns;
	      var &varColumns;
    run;

    %fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end;

	/*==========================================================================* 
	 * Step:            Data Validation                                         *
	 * Transform:       Data Validation                                         * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Transposed Numeric UDFs -                               *
	 *                   work.transposed_numeric_udfs                           * 
	 * Target Table:    Validated Numeric UDFs -                                *
	 *                   work.validated_numeric_udfs                            * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.transposed_numeric_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete validated_numeric_udfs;
    quit;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_errors;
    quit;

    data work.validated_numeric_udfs
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1)
        work.temp_errors
        (keep = ECM_RK VALID_FROM_DTTM _NAME_ COL1);
        set &SOURCEDS end=eof;
        i_move = 0;

        /*---- Custom Validation: No X_SASFCS_RSK_CLSSF  ----*/
        if (_NAME_ = "X_SASFCS_RSK_CLSSF"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_RSK_CLSSF"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /*---- Custom Validation: No X_SASFCS_PTY_KEY  ----*/
        if (_NAME_ = "X_SASFCS_PTY_KEY"  & COL1 = .) then
            do;
                i_move = 1;
            end;  /* if (_NAME_ = "X_SASFCS_PTY_KEY"  & COL1 = .) */
        else
            do;
            end;  /* else */

        /* move record to error file  */
        if (i_move) then
            output work.temp_errors;

        /* move record to output file  */
        else output work.validated_numeric_udfs;

        if eof then
            do;
            end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to validate TRANSPOSED_NUMERIC_UDFS;

        %fcf_put_error;
        %goto MACRO_END;
    %end;



	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    Validated Numeric UDFs -                                *
	 *                   work.validated_numeric_udfs                            * 
	 * Target Table:    New Numeric UDFs -                                      *
	 *                   work.new_numeric_udfs                                  * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.validated_numeric_udfs); 

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_numeric_udfs;
    quit;

	data new_numeric_udfs;
		if 0 then set ecm_db.party_udf_num_value;
		set &SOURCEDS. (keep= ECM_RK _NAME_ COL1 VALID_FROM_DTTM);
		PARTY_RK = ECM_RK;
		VALID_FROM_DTTM = VALID_FROM_DTTM;
		UDF_TABLE_NM = "PARTY";
		UDF_NM = _NAME_;
		ROW_NO = 1;
		UDF_VALUE = COL1;
		drop ECM_RK _NAME_ COL1;
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

    %let SOURCEDS = work.new_numeric_udfs;

	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_UDF_NUM_VALUE -                                   *
	 *                   ecm_db.PARTY_UDF_NUM_VALUE                             * 
	 *                  Join - work.stg_subject_upd                             *
	 * Target Table:    All non-FCF Numeric UDFs -                              *
	 *                   work.non_fcf_numeric_udf                               * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete non_fcf_numeric_udf;
	quit;

	proc sql;
	   create view work.non_fcf_numeric_udf as
	   select
	      PARTY_UDF_NUM_VALUE.PARTY_RK,
	      stg_subject_upd.VALID_FROM_DTTM,
	      PARTY_UDF_NUM_VALUE.UDF_TABLE_NM,
	      PARTY_UDF_NUM_VALUE.UDF_NM,
	      PARTY_UDF_NUM_VALUE.ROW_NO,
	      PARTY_UDF_NUM_VALUE.UDF_VALUE
	   from
	      work.stg_subject_upd, 
	      ecm_db.PARTY_UDF_NUM_VALUE
	   where
	      stg_subject_upd.PARTY_RK = PARTY_UDF_NUM_VALUE.PARTY_RK
	      and stg_subject_upd.OLD_VALID_FROM_DTTM = PARTY_UDF_NUM_VALUE.VALID_FROM_DTTM
	      and PARTY_UDF_NUM_VALUE.UDF_NM NOT IN ('X_SASFCS_RSK_CLSSF','X_SASFCS_PTY_KEY')
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Append                                                  *
	 * Transform:       Append                                                  * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   All non-FCF Numeric UDFs -                              *
	 *                   work.non_fcf_numeric_udf                               * 
	 *                  New Numeric UDFs -                                      *
	 *                   work.new_numeric_udfs                                  * 
	 * Target Table:    All Character UDFs -                                    *
	 *                   work.all_character_udfs                                * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete all_character_udfs;
	quit;

	data work.all_character_udfs / view=work.all_character_udfs;
	   set work.non_fcf_numeric_udf
	       work.new_numeric_udfs;
	   keep PARTY_RK VALID_FROM_DTTM UDF_TABLE_NM UDF_NM ROW_NO UDF_VALUE; 
	run;

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All Character UDFs -                                    *
	 *                   work.all_character_udfs                                * 
	 * Target Table:    PARTY_UDF_NUM_VALUE -                                   *
	 *                   ecm_db.PARTY_UDF_NUM_VALUE                             * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.all_character_udfs); 
	   
	proc append base = ecm_db.PARTY_UDF_NUM_VALUE 
		data = &SOURCEDS force ; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
	   
	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_UDF_LGCHR_VALUE -                                 *
	 *                   ecm_db.PARTY_UDF_LGCHR_VALUE                           * 
	 *                  Join - work.stg_subject_upd                             *
	 * Target Table:    All non-FCF Lgchar UDFs -                               *
	 *                   work.non_fcf_lgchar_udfs                               * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete non_fcf_lgchar_udfs;
	quit;

	proc sql;
	   create table work.non_fcf_lgchar_udfs as
	   select
	      PARTY_UDF_LGCHR_VALUE.PARTY_RK,
	      stg_subject_upd.VALID_FROM_DTTM,
	      PARTY_UDF_LGCHR_VALUE.UDF_TABLE_NM,
	      PARTY_UDF_LGCHR_VALUE.UDF_NM,
	      PARTY_UDF_LGCHR_VALUE.ROW_NO,
	      PARTY_UDF_LGCHR_VALUE.UDF_VALUE
	   from
	      work.stg_subject_upd, 
	      ecm_db.PARTY_UDF_LGCHR_VALUE
	   where
	      stg_subject_upd.PARTY_RK = PARTY_UDF_LGCHR_VALUE.PARTY_RK
	      and stg_subject_upd.OLD_VALID_FROM_DTTM = PARTY_UDF_LGCHR_VALUE.VALID_FROM_DTTM
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All non-FCF Lgchar UDFs -                               *
	 *                   work.non_fcf_lgchar_udfs                               * 
	 * Target Table:    PARTY_UDF_LGCHR_VALUE -                                 *
	 *                   ecm_db.PARTY_UDF_LGCHR_VALUE                           * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.non_fcf_lgchar_udfs); 

	proc append base = ecm_db.PARTY_UDF_LGCHR_VALUE 
		data = &SOURCEDS force ; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
	   
	/*==========================================================================* 
	 * Step:            Join                                                    *
	 * Transform:       Join                                                    * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Tables:   PARTY_UDF_DATE_VALUE -                                  *
	 *                   ecm_db.PARTY_UDF_DATE_VALUE                            * 
	 *                  Join - work.stg_subject_upd                             *
	 * Target Table:    All non-FCF Date UDFs -                                 *
	 *                   work.non_fcf_date_udfs                                 * 
	 *==========================================================================*/ 

	proc datasets lib = work nolist nowarn memtype = (data view);
	   delete non_fcf_date_udfs;
	quit;

	proc sql;
	   create table work.non_fcf_date_udfs as
	   select
	      PARTY_UDF_DATE_VALUE.PARTY_RK,
	      stg_subject_upd.VALID_FROM_DTTM,
	      PARTY_UDF_DATE_VALUE.UDF_TABLE_NM,
	      PARTY_UDF_DATE_VALUE.UDF_NM,
	      PARTY_UDF_DATE_VALUE.ROW_NO,
	      PARTY_UDF_DATE_VALUE.UDF_VALUE
	   from
	      work.stg_subject_upd, 
	      ecm_db.PARTY_UDF_DATE_VALUE
	   where
	      stg_subject_upd.PARTY_RK = PARTY_UDF_DATE_VALUE.PARTY_RK
	      and stg_subject_upd.OLD_VALID_FROM_DTTM = PARTY_UDF_DATE_VALUE.VALID_FROM_DTTM
	   ;
	quit;

	%fcf_rcset(&sqlrc);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 

	/*==========================================================================* 
	 * Step:            Table Loader                                            *
	 * Transform:       Table Loader (Version 2.1)                              * 
	 * Description:                                                             * 
	 *                                                                          * 
	 * Source Table:    All non-FCF Date UDFs -                                 *
	 *                   work.non_fcf_date_udfs                                 * 
	 * Target Table:    PARTY_UDF_DATE_VALUE -                                  *
	 *                   ecm_db.PARTY_UDF_DATE_VALUE                            * 
	 *==========================================================================*/ 

	%let SOURCEDS = %nrquote(work.non_fcf_date_udfs); 

	proc append base = ecm_db.PARTY_UDF_DATE_VALUE 
		data = &SOURCEDS force ; 
	run; 

	%fcf_rcset(&syserr);

	%if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
		%let _fcfmsg_=ERROR: FCF: IEM: Error executing FCF_IEM_UPDATE_SUBJECT;
		%fcf_put_error;
		%goto MACRO_END;
	%end; 
	   
%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
